//*****************************************************************************
//
// File Name	: 'eeprom.c'
// Title		: Itronove hodiny 
#define __PIC18F8720A__
// Author		: http://www.prochazka.zde.cz -> hacesoft 2017
// Created		: 26-09-2017, 15:08
// Revised		: 27-09-2017, 11:27
// Version		: 1.0
// Target MCU	: PIC 18F8720
//
// This code is distributed under the GNU Public License
// Vsechny informace jsou zahrnuty pod GPL licenci, pokud nen explicitne uveden jin typ licence.
// Pouzivn techto strnek ke komercnm celum lze jen se souhlasem autora.
// Vsechna prva vyhrazena (c) 1997 - 2017 hacesoft.
//
//*****************************************************************************

//#include <xc.h>
#include <stddef.h> //nefinuje NULL
#include <stdint.h> //uint8_t, uint16_t, uint32_t
#include <stdio.h>
#include <stdlib.h>

#include "ledI2C.h"
#include "eeprom.h"



//*****************************************************************************
void ReadEEprom (void){
    unsigned int address = 0x0;
    
    PoleLedHSL.IC2_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC2_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC2_PWM.v  = _eeprom_read (address++);
    
    PoleLedHSL.IC3_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC3_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC3_PWM.v  = _eeprom_read (address++);
    
    address = address + 2;
    
    PoleLedHSL.IC4_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC4_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC4_PWM.v  = _eeprom_read (address++);
    
    PoleLedHSL.IC5_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC5_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC5_PWM.v  = _eeprom_read (address++);
    
    address = address + 2;    
    
    PoleLedHSL.IC6_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC6_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC6_PWM.v  = _eeprom_read (address++);
    
    PoleLedHSL.IC7_PWM.h  = _eeprom_read (address++);
    PoleLedHSL.IC7_PWM.s  = _eeprom_read (address++);
    PoleLedHSL.IC7_PWM.v  = _eeprom_read (address++);     
}
//*****************************************************************************
void WriteEEprom (void){
    unsigned int address = 0x0;
    
    _eeprom_write(address++, PoleLedHSL.IC2_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC2_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC2_PWM.v);
    
    _eeprom_write(address++, PoleLedHSL.IC3_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC3_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC3_PWM.v);
    
    address = address + 2;
    
    _eeprom_write(address++, PoleLedHSL.IC4_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC4_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC4_PWM.v);
    
    _eeprom_write(address++, PoleLedHSL.IC5_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC5_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC5_PWM.v);
    
    address = address + 2;    
    
    _eeprom_write(address++, PoleLedHSL.IC6_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC6_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC6_PWM.v);
    
    _eeprom_write(address++, PoleLedHSL.IC7_PWM.h);
    _eeprom_write(address++, PoleLedHSL.IC7_PWM.s);
    _eeprom_write(address++, PoleLedHSL.IC7_PWM.v);
    
}
//*****************************************************************************
uint8_t _eeprom_read (unsigned int address){
    EEADR               =   address; // load address of EEPROM to read
    EECON1bits.EEPGD    =   0; // access EEPROM data memory
    EECON1bits.CFGS     =   0; // do not access configuration registers
    EECON1bits.RD       =   1; // initiate read 
    return EEDATA;              // return EEPROM byte
}
//*****************************************************************************
// Write Byte to internal EEPROM
uint8_t _eeprom_write (unsigned int address, uint8_t data){
    EECON1bits.WREN     =   1;  // allow EEPROM writes
    EEADR               =   address; // load address of write to EEPROM
    EEDATA              =   data; // load data to write to EEPROM
    EECON1bits.EEPGD    =   0;  // access EEPROM data memory
    EECON1bits.CFGS     =   0;  // do not access configuration registers
    INTCONbits.GIE      =   0;  // disable interrupts for critical EEPROM write sequence
    //===============//
    EECON2              =   0x55;
    EECON2              =   0xAA;
    EECON1bits.WR       =   1;
    //==============//
    INTCONbits.GIE      =   1;  // enable interrupts, critical sequence complete
    while (EECON1bits.WR == 1);   // wait for write to complete
    EECON1bits.WREN     =   0;  // do not allow EEPROM writes
    //Verify write operation
    if (_eeprom_read(address) == data) // read the byte we just wrote to EEPROM
    return _true                ; // write was successful
    else
    return _false;                  // write failed
}
//*****************************************************************************

 



