//*****************************************************************************
//
// File Name	: 'hsv.c'
// Title		: Itronove hodiny 
#define __PIC18F8720A__
// Author		: http://www.prochazka.zde.cz -> hacesoft 2017
// Created		: 15-06-2017, 07:03
// Revised		: 15-06-2017, 07:03
// Version		: 1.0
// Target MCU	: PIC 18F8720
//
// This code is distributed under the GNU Public License
// Vsechny informace jsou zahrnuty pod GPL licenci, pokud nen explicitne uveden jin typ licence.
// Pouzivn techto strnek ke komercnm celum lze jen se souhlasem autora.
// Vsechna prva vyhrazena (c) 1997 - 2017 hacesoft.
//
//*****************************************************************************
//https://stackoverflow.com/questions/3018313/algorithm-to-convert-rgb-to-hsv-and-hsv-to-rgb-in-range-0-255-for-both

#include "hsv.h"
#include <xc.h>

RgbColor HsvToRgb(HsvColor hsv){
    RgbColor rgb;
    unsigned char region, p, q, t;
    unsigned int h, s, v, remainder;

    if (hsv.s == 0){
        rgb.r = hsv.v;
        rgb.g = hsv.v;
        rgb.b = hsv.v;
        return rgb;
    }

    // converting to 16 bit to prevent overflow
    h = hsv.h;
    s = hsv.s;
    v = hsv.v;

    region = h / 43;
    remainder = (h - (region * (unsigned)43)) * 6; 

    p = (v * (255 - s)) >> 8;
    q = (v * (255 - ((s * remainder) >> 8))) >> 8;
    t = (v * (255 - ((s * (255 - remainder)) >> 8))) >> 8;

    switch (region){
        case 0:
            rgb.r = v;
            rgb.g = t;
            rgb.b = p;
            break;
        case 1:
            rgb.r = q;
            rgb.g = v;
            rgb.b = p;
            break;
        case 2:
            rgb.r = p;
            rgb.g = v;
            rgb.b = t;
            break;
        case 3:
            rgb.r = p;
            rgb.g = q;
            rgb.b = v;
            break;
        case 4:
            rgb.r = t;
            rgb.g = p;
            rgb.b = v;
            break;
        default:
            rgb.r = v;
            rgb.g = p;
            rgb.b = q;
            break;
    }

    return rgb;
}

HsvColor RgbToHsv(RgbColor rgb){
    HsvColor hsv;
    unsigned char rgbMin, rgbMax;

    rgbMin = (unsigned)rgb.r < (unsigned)rgb.g ? (unsigned)(rgb.r < rgb.b ? rgb.r : rgb.b) : (unsigned)(rgb.g < rgb.b ? rgb.g : rgb.b);
    rgbMax = (unsigned)rgb.r > (unsigned)rgb.g ? (unsigned)(rgb.r > rgb.b ? rgb.r : rgb.b) : (unsigned)(rgb.g > rgb.b ? rgb.g : rgb.b);

    hsv.v = rgbMax;
    if (hsv.v == 0){
        hsv.h = 0;
        hsv.s = 0;
        return hsv;
    }

    hsv.s = 255 * (unsigned)((long)(rgbMax - rgbMin)) / hsv.v;
    if (hsv.s == 0){
        hsv.h = 0;
        return hsv;
    }

    if (rgbMax == rgb.r)
        hsv.h = 0 + 43 * (unsigned)(rgb.g - rgb.b) / (unsigned)(rgbMax - rgbMin);
    else if (rgbMax == rgb.g)
        hsv.h = 85 + 43 * (unsigned)(rgb.b - rgb.r) / (unsigned)(rgbMax - rgbMin);
    else
        hsv.h = 171 + 43 * (unsigned)(rgb.r - rgb.g) / (unsigned)(rgbMax - rgbMin);

    return hsv;
}
  