//*****************************************************************************
//
// File Name	: 'i2c.c'
// Title		: Itronove hodiny 
#define __PIC18F8720A__
// Author		: http://www.prochazka.zde.cz -> hacesoft 2017
// Created		: 30-06-2017, 11:58
// Revised		: 04-07-2017, 08:52
// Version		: 1.3
// Target MCU	: PIC 18F8720
//
// This code is distributed under the GNU Public License
// Vsechny informace jsou zahrnuty pod GPL licenci, pokud nen explicitne uveden jin typ licence.
// Pouzivn techto strnek ke komercnm celum lze jen se souhlasem autora.
// Vsechna prva vyhrazena (c) 1997 - 2017 hacesoft.
//
//*****************************************************************************

#include <xc.h>
#include <stdint.h> //uint8_t, uint16_t, uint32_t
#include "i2c.h"

//*****************************************************************************
void I2C_Start(uint8_t type){
    if (type == _SW){I2C_StartMasterSW();}
    else i2c_Start();
}
//*****************************************************************************
void I2C_StartMasterSW(void){
    i2c_WaitGeneral();
	SDA_MasterSW = 1;		// Pull SDA High
    __delay_us(_wt_speed_i2c_Master);
    SCL_MasterSW = 1;		// Pull SCL High
	__delay_us(_wt_speed_i2c_Master);
	SDA_MasterSW = 0;		// Pull SDA Low
	__delay_us(_wt_speed_i2c_Master);
    SCL_MasterSW = 0;
    i2c_WaitGeneral();
}
//*****************************************************************************
// i2c_Start - Start I2C communication
void i2c_Start(void){
    GIE = 0;        // Global interrupt stop
    i2c_WaitGeneral();
    I2C_HW_Wait();
    SEN=1;
    i2c_WaitGeneral();
}
//*****************************************************************************
// i2c_Start - Start I2C communication
void I2C_Start_PCA955PW(void){
    GIE = 0;        // Global interrupt stop
    i2c_WaitGeneral_PCA955PW();
    I2C_HW_Wait_PCA955PW();
    SEN=1;
    i2c_WaitGeneral_PCA955PW();
}
//*****************************************************************************
void I2C_Stop(uint8_t type){
    if (type == _SW){I2C_StopMasterSW();}
    else i2c_Stop();
}
//*****************************************************************************
void I2C_StopMasterSW(void){
    i2c_WaitGeneral();
    SCL_MasterSW = 1;
	__delay_us(_wt_speed_i2c_Master);
	SDA_MasterSW = 1;         
	i2c_WaitGeneral();
}
//*****************************************************************************
// i2c_Stop - Stop I2C communication
void i2c_Stop(void){
    i2c_WaitGeneral();
    I2C_HW_Wait();
    PEN=1;
    i2c_WaitGeneral();
    GIE = 1;        // Global interrupt enable
}
//*****************************************************************************
// i2c_Stop - Stop I2C communication
void I2C_Stop_PCA955PW(void){
    i2c_WaitGeneral_PCA955PW();
    I2C_HW_Wait_PCA955PW();
    PEN=1;
    i2c_WaitGeneral_PCA955PW();
    GIE = 1;        // Global interrupt enable
}
//*****************************************************************************
void I2C_Write(uint8_t type, unsigned char data){
    if (type == _SW){I2C_WriteMasterSW(data);}
    else i2c_Write(data);
}
//*****************************************************************************
void I2C_WriteMasterSW(unsigned char data){
	uint8_t i;
	uint8_t SdaBuffer;
	
	for(i=0;i<8;i++){						// loop 8 times to send 1-byte of data
        i2c_WaitGeneral();
		SdaBuffer = data & (unsigned) 0x80;				// Send Bit by Bit on SDA line
            if (SdaBuffer == 0x80){SDA_MasterSW = 1;}
            else {SDA_MasterSW = 0;} 
		I2C_ClockMasterSW();					// Generate Clock at SCL
		data = (unsigned) data << 1;
	}
    i2c_WaitGeneral();
    SDA_MasterSW = 1;                         // Set SDA at last
    i2c_WaitGeneral();
    I2C_AckNowLedgeMasterSW();
}
//*****************************************************************************
// i2c_Write - Sends one byte of data
void i2c_Write(unsigned char data){
    I2C_HW_Wait();
    SSPBUF = data;
    i2c_WaitGeneral();
}
//*****************************************************************************
// i2c_Write - Sends one byte of data
void I2C_Write_PCA955PW(unsigned char data){
    I2C_HW_Wait_PCA955PW();
    SSPBUF = data;
    i2c_WaitGeneral_PCA955PW();
}
//*****************************************************************************
// i2c_Address - Sends Slave Address and Read/Write mode
// mode is either I2C_WRITE or I2C_READ
void i2c_Address(unsigned char address, unsigned char mode){
    unsigned char l_address;

    l_address = (unsigned) address << 1;
    l_address+=mode;
    I2C_HW_Wait();
    SSPBUF = l_address;
    i2c_WaitGeneral();
}
//*****************************************************************************
unsigned char I2C_Read(uint8_t type, unsigned char ack){
    unsigned char nBuffer;
    
    if (type == _SW){ nBuffer = I2C_ReadMasterSW();}
    else nBuffer = i2c_Read(ack);
   return nBuffer;
}
//*****************************************************************************
//I2C_Read()
//I/P Arguments: none
//Return value : Unsigned char(received byte)
//Description :This fun is used to receive a byte on SDA line using I2C protocol.
//8bit data is received bit-by-bit each clock and finally packed into Byte.
//MSB(bit) is received first and LSB(bit) is received at last.
//I2C read.png
unsigned char I2C_ReadMasterSW(void){
	unsigned char i = 0; 
    unsigned char nBuffer = 0x00;
	
	SDA_MasterSW = 1;                   //Make SDA as I/P
	for(i=0;i<8;i++){                    // loop 8times to read 1-byte of data
		i2c_WaitGeneral();
         SCL_MasterSW = 1;               //Pull SCL High
        i2c_WaitGeneral();
		nBuffer = (unsigned)nBuffer << 1;        //dat is Shifted each time and
		nBuffer = (unsigned)nBuffer | SDA_MasterSW;		//ORed with the received bit to pack into byte
		//dat = dat | (PIND, SDAbit);	//ORed with the received bit to pack into byte
		SCL_MasterSW = 0;                // Clear SCL to complete the Clock
        i2c_WaitGeneral();
	}
	return (nBuffer);                         // Finally return the received Byte*
}
//*****************************************************************************
// i2c_Read - Reads a byte from Slave device
unsigned char i2c_Read(unsigned char ack){
    // Read data from slave
    // ack should be 1 if there is going to be more data read
    // ack should be 0 if this is the last byte of data read
    // ack nastavit na 1 v p?pad?, ?e se ma cist vice dat ...
    // ack nastavit na 0, pokud se jedna o posledni bajt ...
    unsigned char i2cReadData;

    I2C_HW_Wait();
    RCEN=1;
    I2C_HW_Wait();
    i2cReadData = SSPBUF;
    I2C_HW_Wait();
    if ( ack ) ACKDT=0;	        // Ack
    else       ACKDT=1;	        // NAck
    ACKEN=1;                    // send acknowledge sequence
    return( i2cReadData );
}
//*****************************************************************************
// i2c_Read - Reads a byte from Slave device
unsigned char I2C_Read_PCA955PW(unsigned char ack){
    // Read data from slave
    // ack should be 1 if there is going to be more data read
    // ack should be 0 if this is the last byte of data read
    // ack nastavit na 1 v p?pad?, ?e se ma cist vice dat ...
    // ack nastavit na 0, pokud se jedna o posledni bajt ...
    unsigned char i2cReadData;

    I2C_HW_Wait_PCA955PW();
    RCEN=1;
    I2C_HW_Wait_PCA955PW();
    i2cReadData = SSPBUF;
    I2C_HW_Wait_PCA955PW();
    if ( ack ) ACKDT=0;	        // Ack
    else       ACKDT=1;	        // NAck
    ACKEN=1;                    // send acknowledge sequence
    return( i2cReadData );
}
//*****************************************************************************
void i2c_Ack_HW(unsigned char ack){
    i2c_WaitGeneral();
    if ( ack ) ACKDT=0;	        // Ack
    else       ACKDT=1;	        // NAck
    I2C_HW_Wait();
    ACKEN=1;                    // send acknowledge sequence
    i2c_WaitGeneral();
}
//*****************************************************************************
void I2C_AckNowLedgeMasterSW(void){
    //i2c_WaitGeneral();
	////__delay_us(_wt_speed_i2c_Master);		// Wait for Some time
	SCL_MasterSW = 1; 
	__delay_us(_wt_speed_i2c_Master);		// Wait for Some time
	SCL_MasterSW = 0;
    //i2c_WaitGeneral();
    __delay_us(_wt_speed_i2c_Master);
}
//*****************************************************************************
void I2C_ReStart(uint8_t type){
    if (type == _SW){ I2C_ReStartMasterSW();}
    else i2c_Restart(); 
}
//*****************************************************************************
void I2C_ReStartMasterSW (void){
    //i2c_WaitGeneral();
    __delay_us(_wt_speed_i2c_Master);
    SDA_MasterSW = 1;
    SCL_MasterSW = 0;
    __delay_us(_wt_speed_i2c_Master);
    SCL_MasterSW = 1;
    __delay_us(_wt_speed_i2c_Master);
    SDA_MasterSW = 0;
    __delay_us(_wt_speed_i2c_Master);
    SCL_MasterSW = 0;
    __delay_us(_wt_speed_i2c_Master);
}
//*****************************************************************************
// i2c_Restart - Re-Start I2C communication
void i2c_Restart(void){
    i2c_WaitGeneral();
    I2C_HW_Wait();
    RSEN=1;
    i2c_WaitGeneral();
}
//*****************************************************************************
// i2c_Restart - Re-Start I2C communication
void I2C_ReStart_PCA955PW(void){
    i2c_WaitGeneral_PCA955PW();
    I2C_HW_Wait_PCA955PW();
    RSEN=1;
    i2c_WaitGeneral_PCA955PW();
}
//*****************************************************************************
// i2c_Wait - wait for I2C transfer to finish
void I2C_HW_Wait_PCA955PW(void){
    __delay_ms(2);
    while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F)); //Transmit is in progress
}
//*****************************************************************************
void i2c_WaitGeneral_PCA955PW(void){
    __delay_us(_wt_speed_i2c_Master * 2);
}
//*****************************************************************************
// i2c_Wait - wait for I2C transfer to finish
void I2C_HW_Wait(void){
    __delay_ms(2);
    while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F)); //Transmit is in progress

}
//*****************************************************************************
void i2c_WaitGeneral(void){
    __delay_us(_wt_speed_i2c_Master * 2);
}
//*****************************************************************************
void I2C_ClockMasterSW(void){
    SCL_MasterSW = 0; 
	__delay_us(_wt_speed_i2c_Master);		// Wait for Some time
	SCL_MasterSW = 1;  
	__delay_us(_wt_speed_i2c_Master);		// Wait for Some time
	SCL_MasterSW = 0; 
    __delay_us(_wt_speed_i2c_Master);		// Wait for Some time
}
//*****************************************************************************
void InitMasterPort(uint8_t type){
    if (type == _SW){ InitMasterSwPort();}
    else InitMasterHwPort(); 
}
//*****************************************************************************

void InitMasterSwPort(void){
    BitClear (PORTB,SCL_MasterSW_PIN);
    BitClear (PORTB,SDA_MasterSW_PIN);
    BitClear (LATB,SCL_MasterSW_PIN);
    BitClear (LATB,SDA_MasterSW_PIN);
    SDAt_MasterSW = Output;				// SDA Output		
    SCLt_MasterSW = Output;				// SCL Output		
    SDA_MasterSW  = 1;
    SCL_MasterSW  = 1;
}
//*****************************************************************************
void InitMasterHwPort(void){
   BitClear (PORTC,SCL_MasterHW_PIN);
   BitClear (PORTC,SDA_MasterHW_PIN);
   BitClear (LATC,SCL_MasterHW_PIN);
   BitClear (LATC,SDA_MasterHW_PIN);
   
   SCLt_MasterHW = Input;				// SCL Input		
   SDAt_MasterHW = Input;				// SDA Input		

   SSPCON1 = 0b00101000; 	// I2C enabled, Master mode
   SSPCON2 = 0x00;
   SSPADD = 254;           // 100kHz bus with 20Mhz xtal

}
