//*****************************************************************************
//
// File Name	: 'ledI2C.c'
// Title		: Itronove hodiny 
#define __PIC18F8720A__
// Author		: http://www.prochazka.zde.cz -> hacesoft 2017
// Created		: 24-05-2017, 13:38
// Revised		: 24-05-2017, 14:09
// Version		: 1.0
// Target MCU	: PIC 18F8720
//
// This code is distributed under the GNU Public License
// Vsechny informace jsou zahrnuty pod GPL licenci, pokud nen explicitne uveden jin typ licence.
// Pouzivn techto strnek ke komercnm celum lze jen se souhlasem autora.
// Vsechna prva vyhrazena (c) 1997 - 2017 hacesoft.
//
//*****************************************************************************


#include <xc.h>
#include "hsv.h"
#include "i2c.h"
#include <stdint.h> //uint8_t, uint16_t, uint32_t
#include "ledI2C.h"



void InitIO_PCA9633_HW(uint8_t IO_adress){
    
    I2C_Start(_HW);
    
    I2C_Write(_HW,IO_adress);   //write slave address
    // MODE1 - Mode register 1 (address 00h) bit description
    I2C_Write(_HW,0b10000000);  //pocatecni adresa registru 0x0
    // vynulovat registr
    I2C_Write(_HW,0b00000000); 
    
    I2C_ReStart(_HW);
    
    I2C_Write(_HW,IO_adress);  //write slave address ALL
    
    // Control register
    //Auto-Increment for all registers. D[4:0] roll over to ?0 0000? after the last register (1 1011) is accessed.
    I2C_Write(_HW,0b10000110); //pocatecni adresa registru 0x6
    
    // Reg 0x6 - Skupina PWM - by nevad
    //GRPPWM - Group brightness control register (address 12h) bit description
    I2C_Write(_HW,0xFF);  
    
    // Reg 0x7 - Skupina Freq - by nevad
    //GRPFREQ - Group Frequency register (address 13h) bit description
    I2C_Write(_HW,0x00);
    
    // Reg 0x14-0x17 - LED vstupu sttn kontrole - v?echny 0xAA (stav vstupu pro ka?dou LED = 0x2)LDRx = 00 ? LED driver x is off (default power-up state).
    //LDRx = 01 ? LED driver x is fully on (individual brightness and group dimming/blinking not controlled).
    //LDRx = 10 ? LED driver x individual brightness can be controlled through its PWMx register.
    //LDRx = 11 ? LED driver x individual brightness and group dimming/blinking can be controlled through its PWMx register and the GRPPWM registers.
    I2C_Write(_HW,0b10101010);  //LEDOUT register (LED fully ON)
    
    I2C_Stop(_HW); 
}
//*****************************************************************************

void InitIO_PCA9633_SW(uint8_t IO_adress){
    I2C_Start(_SW);
    
    I2C_Write(_SW,IO_adress);  //write slave address
    // MODE1 - Mode register 1 (address 00h) bit description
    I2C_Write(_SW,0b10000000);
  
    // vynulovat registr
    I2C_Write(_SW,0b00000000);
    
    I2C_ReStart(_SW);
    I2C_Write(_SW,IO_adress);  //write slave address
    
    // Control register
    //Auto-Increment for all registers. D[4:0] roll over to ?0 0000? after the last register (1 1011) is accessed.
    I2C_Write(_SW,0b10000110);//pocatecni adresa registru 0x6
  
    // Reg 0x6 - Skupina PWM - by nevad
    //GRPPWM - Group brightness control register (address 12h) bit description
    I2C_Write(_SW,0xFF);

    // Reg 0x7 - Skupina Freq - by nevad
    //GRPFREQ - Group Frequency register (address 13h) bit description
    I2C_Write(_SW,0x00);

    // Reg 0x14-0x17 - LED vstupu sttn kontrole - v?echny 0xAA (stav vstupu pro ka?dou LED = 0x2)LDRx = 00 ? LED driver x is off (default power-up state).
    //LDRx = 01 ? LED driver x is fully on (individual brightness and group dimming/blinking not controlled).
    //LDRx = 10 ? LED driver x individual brightness can be controlled through its PWMx register.
    //LDRx = 11 ? LED driver x individual brightness and group dimming/blinking can be controlled through its PWMx register and the GRPPWM registers.
    I2C_Write(_SW,0b10101010);
    
   I2C_Stop(_SW); 
}
//*****************************************************************************
void setRGB_PCA9633(uint8_t type,uint8_t IO_adress, uint8_t R, uint8_t G, uint8_t B){
    if (type == _SW){setRGB_PCA9633_SW(IO_adress,R,G,B);}
    else setRGB_PCA9633_HW(IO_adress,R,G,B);
}
//*****************************************************************************
void setRGB_PCA9633_SW(uint8_t IO_adress, uint8_t R, uint8_t G, uint8_t B){
    //uint8_t PWMa = 0b10100000 | Reg;
    I2C_Start(_SW);
    I2C_Write(_SW,IO_adress);  //write slave address ALL
    I2C_Write(_SW,0b10100011); //pocatecni adresa PWM registru je PWM1
    I2C_Write(_SW,R);
    I2C_Write(_SW,G);
    I2C_Write(_SW,B);

    I2C_Stop(_SW); 
}
//*****************************************************************************
void setRGB_PCA9633_HW(uint8_t IO_adress, uint8_t R, uint8_t G, uint8_t B){
    //uint8_t PWMa = 0b10100000 | Reg;
    I2C_Start(_HW);
    I2C_Write(_HW,IO_adress);  //write slave address ALL
    I2C_Write(_HW,0b10100011); //pocatecni adresa PWM registru je PWM1
    I2C_Write(_HW,R);
    I2C_Write(_HW,G);
    I2C_Write(_HW,B);

    I2C_Stop(_HW); 
}
//*****************************************************************************
int ABS(int a){
	if(a < 0)
		return -a;
	return a;
}
//*****************************************************************************
void SetLightHua (void){
    RgbColor rgb;

    rgb = HsvToRgb(PoleLedHSL.IC2_PWM);
    setRGB_PCA9633(_HW,adress_LED_IC2, rgb.r, rgb.g, rgb.b);
    rgb = HsvToRgb(PoleLedHSL.IC3_PWM);
    setRGB_PCA9633(_HW,adress_LED_IC3, rgb.r, rgb.g, rgb.b);
    rgb = HsvToRgb(PoleLedHSL.IC4_PWM);
    setRGB_PCA9633(_HW,adress_LED_IC4, rgb.r, rgb.g, rgb.b);
    rgb = HsvToRgb(PoleLedHSL.IC5_PWM);
    setRGB_PCA9633(_HW,adress_LED_IC5, rgb.r, rgb.g, rgb.b);
    rgb = HsvToRgb(PoleLedHSL.IC6_PWM);
    setRGB_PCA9633(_SW,adress_LED_IC6, rgb.r, rgb.g, rgb.b);
    rgb = HsvToRgb(PoleLedHSL.IC7_PWM);
    setRGB_PCA9633(_SW,adress_LED_IC7, rgb.r, rgb.g, rgb.b);  
}
//*****************************************************************************
