//*****************************************************************************
//
// File Name	: 'rtc.c'
// Title		: Itronove hodiny 
#define __PIC18F8720A__
// Author		: http://www.prochazka.zde.cz -> hacesoft 2017
// Created		: 05-05-2017, 15:06
// Revised		: 07-09-2017, 10:16
// Version		: 1.1
// Target MCU	: PIC 18F8720
//
// This code is distributed under the GNU Public License
// Vsechny informace jsou zahrnuty pod GPL licenci, pokud nen explicitne uveden jin typ licence.
// Pouzivn techto strnek ke komercnm celum lze jen se souhlasem autora.
// Vsechna prva vyhrazena (c) 1997 - 2017 hacesoft.
//
//*****************************************************************************

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include "i2c.h"
#include "display.h"
#include <stdint.h> //uint8_t, uint16_t, uint32_t
#include "rtc.h"
//*****************************************************************************
void ReadRTC(void) {
    __delay_ms(5);
    I2C_Start(_HW);
    __delay_ms(5);
    I2C_Write(_HW, AddressDS3231_write); //write slave address
    __delay_ms(5);
    I2C_Write(_HW, 0); //zaciname registrem 0
    __delay_ms(5);
    
    I2C_ReStart(_HW);
    __delay_ms(5);
    I2C_Write(_HW, AddressDS3231_read); //write slave address
    __delay_ms(5);
    ParseSecond(I2C_Read(_HW, ACKNOWLEDGE)); //Seconds
    ParseMinutes(I2C_Read(_HW, ACKNOWLEDGE)); //Minutes 
    ParseHour(I2C_Read(_HW, ACKNOWLEDGE)); //Hour 
    _DateTimeDS3231.Day = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Date = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Month_Century = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Year = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_1_Seconds = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_1_Minutes = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_1_Hours = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_1_Day_Date = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_2_Minutes = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_2_Hours = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Alarm_2_Day_Date = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Control = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Control_Status = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.Aging_Offset = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.MSB_buffer_temp = I2C_Read(_HW, ACKNOWLEDGE);
    _DateTimeDS3231.LSB_buffer_temp = I2C_Read(_HW, NOT_ACKNOWLEDGE);
    I2C_Stop(_HW);
    __delay_ms(5);
}
//*****************************************************************************
void SetTime(void) {
    __delay_ms(5);
    I2C_Start(_HW);
    __delay_ms(5);
    I2C_Write(_HW, AddressDS3231_write); //write slave address
    I2C_Write(_HW, 0x0); //zaciname registrem 0
    I2C_Write(_HW, CodeSeconds (_DateTimeDS3231.Seconds_10, _DateTimeDS3231.Seconds));
    I2C_Write(_HW, CodeMinutes (_DateTimeDS3231.Minutes_10, _DateTimeDS3231.Minutes));
    I2C_Write(_HW, CodeHours (_DateTimeDS3231.Hour_10, _DateTimeDS3231.Hour));
    I2C_Stop(_HW);
    __delay_ms(5);
}
//*****************************************************************************
void SetDate(void) {
    __delay_ms(5);
    I2C_Start(_HW);
    __delay_ms(5);
    I2C_Write(_HW, AddressDS3231_write); //write slave address
    I2C_Write(_HW, 0x3); //zaciname registrem 3
    I2C_Write(_HW, CodeDay (_DateTimeDS3231.Day));
    I2C_Write(_HW, CodeDate (_DateTimeDS3231.Date));
    I2C_Write(_HW, CodeMonth_Century (_DateTimeDS3231.Month_Century));
    I2C_Write(_HW, CodeYear(_DateTimeDS3231.Year));
    I2C_Stop(_HW);
    __delay_ms(5);
}
//*****************************************************************************
float GetTemp(void) {
    uint8_t _msb = _DateTimeDS3231.MSB_buffer_temp;
    uint8_t _lsb = _DateTimeDS3231.LSB_buffer_temp;
    uint16_t _nBuffer;
    _nBuffer = (unsigned)((unsigned)_msb rol 2) or ((unsigned)_lsb ror 6);
    return (float) _nBuffer * 0.25;
}
//*****************************************************************************
uint8_t CodeSeconds ( uint8_t Seconds_10, uint8_t  Seconds) {
 
    return (unsigned)Seconds or ((unsigned)Seconds_10 rol 4);
}
//*****************************************************************************
uint8_t CodeMinutes (uint8_t Minutes_10, uint8_t Minutes){
   
    return (unsigned) Minutes or ((unsigned)Minutes_10 rol 4);
}
//*****************************************************************************
uint8_t CodeHours (uint8_t Hour_10,uint8_t Hour) {
    uint8_t nBuffer;
    
    nBuffer = (unsigned)Hour or ((unsigned)Hour_10 rol 4);
    BitClear (nBuffer, 6); //Nastaveni rezimu 24 hodin
    //Bit 6 of the hours register is defined as the 12- or 24-hour mode select bit. 
        //When high, the 12-hour mode is selected. In the 12-hour mode, bit 5 is the AM/PM bit with logic-high being PM. 
        //In the 24-hour mode, bit 5 is the 20-hour bit (20?23 hours). 
    return nBuffer;
}
//*****************************************************************************
uint8_t CodeDay (uint8_t Day) {

    return _DateTimeDS3231.Day;    
}
//*****************************************************************************
uint8_t CodeDate (uint8_t Date){

    return _DateTimeDS3231.Date;    
}
//*****************************************************************************
uint8_t CodeMonth_Century (uint8_t Month_Century){

    return _DateTimeDS3231.Month_Century;    
}
//*****************************************************************************
uint8_t CodeYear (uint8_t _Year){
 
    return _DateTimeDS3231.Year;
}
//*****************************************************************************

void ShowTemp(void) {
    char *buff;
    uint8_t sBuffer;
    uint8_t sBuffer1;
    uint8_t a;

    buff = _ftoa(GetTemp());
    a = 0;
    sBuffer = buff[a];
    if (sBuffer == minus) {
        a++;
        _hour_raw(decode_segment(21), decode_segment(16));
    }
    sBuffer = buff[a];
    a++;
    sBuffer1 = buff[a];
    if (sBuffer1 == tecka) {
        _minut_raw(decode_segment(21), (unsigned) (decode_segment((unsigned) sBuffer - 48) or decode_segment(20)));
    } else {
        _minut_raw(decode_segment((unsigned) sBuffer - 48),(unsigned) (decode_segment((unsigned) sBuffer1 - 48) or decode_segment(20)));
        a++;
    }
    _second_raw(decode_segment((unsigned)buff[a+1] - 48), decode_segment((unsigned)buff[a+2] - 48));
}
//*****************************************************************************
char* _ftoa(float f) {
    static char buf[17];
    char* cp = buf;
    unsigned long l, rem;

    if (f < 0) {
        *cp++ = '-';
        f = -f;
    }
    l = (unsigned long) f;
    f -= (float) l;
    rem = (unsigned long) (f * 1e2);
    sprintf(cp, "%lu.%2.2lu", l, rem);
    return buf;
}
//*****************************************************************************
void init_RTC(void) {
    init_RST_RTC();
    init_INT_RTC();
    ResetRTC();
}
//*****************************************************************************
void init_RST_RTC(void) {
    //CLRF PORTB ; Initialize PORTB by
    //; clearing output
    //; data latches
    //CLRF LATB ; Alternate method
    //; to clear output
    //; data latches
    //MOVLW 0xCF ; Value used to
    //; initialize data
    //; direction
    //MOVWF TRISB ; Set RB<3:0> as inputs
    //; RB<5:4> as outputs
    //; RB<7:6> as inputs
    //BitClear(RST_RTC, _RB4);
    BitSet(RST_RTC, _RB4);
    BitClear(LATB_RTC, _RB4);
    BitClear(tRTC, _RB4);
    BitSet(RST_RTC, _RB4);
}
//*****************************************************************************
void init_INT_RTC(void) {
    //CLRF PORTB ; Initialize PORTB by
    //; clearing output
    //; data latches
    //CLRF LATB ; Alternate method
    //; to clear output
    //; data latches
    //MOVLW 0xCF ; Value used to
    //; initialize data
    //; direction
    //MOVWF TRISB ; Set RB<3:0> as inputs
    //; RB<5:4> as outputs
    //; RB<7:6> as inputs
    BitSet(INT_RTC, _RB3);
    BitSet(LATB_RTC, _RB3);
    BitSet(tRTC, _RB3);
}
//*****************************************************************************
void ResetRTC(void) {
    BitSet(RST_RTC, _RB4);
    __delay_ms(10);
    BitClear(RST_RTC, _RB4);
    __delay_ms(10);
    BitSet(RST_RTC, _RB4);
}
//*****************************************************************************
void ParseSecond(unsigned char ParseData) {
    _DateTimeDS3231.Seconds = (unsigned) ParseData and MaskSeconds;
    _DateTimeDS3231.Seconds_10 = (unsigned) (ParseData and MaskSeconds_10) >> 4;
}
//*****************************************************************************
void ParseMinutes(unsigned char ParseData) {
    _DateTimeDS3231.Minutes = (unsigned) ParseData and MaskMinutes;
    _DateTimeDS3231.Minutes_10 = (unsigned) (ParseData and MaskMinutes_10) >> 4;
}
//*****************************************************************************
void ParseHour(unsigned char ParseData) {
    _DateTimeDS3231.Hour = (unsigned) ParseData and MaskHour;
    _DateTimeDS3231.Hour_10 = (unsigned) (ParseData and MaskHour_10) >> 4;
    _DateTimeDS3231.AM_PM = (unsigned) (ParseData and MaskAM_PM) >> 5;
    _DateTimeDS3231._12_24 = (unsigned) (ParseData and Mask_12_24) >> 6;
}
//*****************************************************************************
